@@ -1,4 +1,3 @@
-import warnings
 from typing import List, Dict, Optional
 from quid.core.InternalMatch import InternalMatch
 from quid.core.BestMatch import BestMatch
@@ -8,8 +7,10 @@ from quid.match.MatchSpan import MatchSpan
 from quid.core.Text import Text
 import re
 from quid.core.Token import Token
-from rapidfuzz import fuzz, process
+from rapidfuzz import process
+from rapidfuzz.distance import Levenshtein
 from datasketch import MinHash, MinHashLSH
+import warnings
 
 
 # noinspection PyMethodMayBeStatic
@@ -22,14 +23,16 @@ class Quid:
     forward_references: Dict[int, List[int]]
     # Values relevant for fuzzy matching
     HASH_PERM: int = 128
-    SCORE_CUTOFF_SHORT_STRINGS: int = 85
-    SCORE_CUTOFF: int = 90
-    LSH_THRESHOLD: float = 0.95
+    SCORE_CUTOFF_SHORT_STRINGS: int = 0.80
+    SCORE_CUTOFF: int = 0.85
+    LSH_THRESHOLD: float = 0.70
 
-    SENTENCE_DELIMITER = '\u03B1'
+    SENTENCE_DELIMITER = '\u2190'
     SENTENCE_DELIMITER_START_REGEX: str = f'^[{SENTENCE_DELIMITER}]'
     SENTENCE_DELIMITER_END_REGEX: str = f'[{SENTENCE_DELIMITER}]$'
 
+    RESERVED_CHARACTERS = ['\u2190', '\u2191']
+
     def __init__(self, min_match_length: int = 5,
                  look_back_limit: int = 10,
                  look_ahead_limit: int = 3,
@@ -50,10 +53,6 @@ class Quid:
         if min_match_length < 1:
             raise ValueError('min match length must be >= 1')
 
-        if min_match_length < 3:
-            warnings.warn('A small minimal match length can lead to long processing times '
-                          '(recommendation: min_match_length >= 3)')
-
         if look_back_limit < 0:
             raise ValueError('look back limit must be positive')
 
@@ -134,13 +133,10 @@ class Quid:
         tokens: List[Token] = []
 
         for input_text in input_texts:
-            nr_of_characters = len(input_text)
-            nr_of_words = len(input_text.split())
             tk_start_pos = len(tokens)
-
             tokens.extend(self.__tokenize_text(input_text))
             tk_end_pos = len(tokens)
-            text = Text(nr_of_characters, nr_of_words, tk_start_pos, tk_end_pos)
+            text = Text(tk_start_pos, tk_end_pos)
             texts.append(text)
 
         return texts, tokens
@@ -160,10 +156,24 @@ class Quid:
         return tokens
 
     def __clean_text(self, input_text: str) -> str:
+
+        for char in self.RESERVED_CHARACTERS:
+            if char in input_text:
+                warnings.warn(f'Text contains reserved character {char}. This might lead to unwanted behaviour.')
+
         input_text = re.sub('(\\[\\.\\.\\.]|\\[…]|\\.\\.\\.|…)', lambda x: '@' * len(x.group(1)), input_text)
         input_text = re.sub('[.;!?]', self.SENTENCE_DELIMITER, input_text)
-        input_text = re.sub(f'[^a-zA-Z\\däüöÄÜÖß@{self.SENTENCE_DELIMITER} ]', ' ', input_text)
+
+        # preserve [s], [n] [er ]etc.
+        # input_text = re.sub(r'(\w)\[([a-zA-Z])]', '\\g<1>\u2191\\g<2>\u2191', input_text)
+        input_text = re.sub(r'\[([a-zA-Z]{1,3})]', '\u2191\\g<1>\u2191', input_text)
+
+        input_text = re.sub(rf'[^\w_@{self.SENTENCE_DELIMITER}\u2191 ]', ' ', input_text)
         input_text = re.sub(r'\d', ' ', input_text)
+
+        # input_text = re.sub(r'(\w)\u2191([a-zA-Z])\u2191', '\\g<1>[\\g<2>]', input_text)
+        input_text = re.sub(r'\u2191([a-zA-Z]{1,3})\u2191', '[\\g<1>]', input_text)
+
         return input_text.lower()
 
     def __clean_word(self, input_word: str) -> str:
@@ -172,13 +182,15 @@ class Quid:
         input_word = input_word.replace('ö', 'oe')
         input_word = input_word.replace('ü', 'ue')
         input_word = input_word.replace('ey', 'ei')
+        input_word = input_word.replace('[', '')
+        input_word = input_word.replace(']', '')
         return input_word
 
     def __remove_special_characters(self, input_string: str) -> str:
+        input_string = re.sub(r'[^\w@ ]|[\u03B1\u03B2]', '', input_string)
 
-        input_string = re.sub(r'[^a-zA-Z\däüöÄÜÖß@ ]', '', input_string)
-
-        if re.search(r'[a-zA-Z\däüöÄÜÖß]', input_string):
+        # TODO: _ ist bei \w dabei, ist das ein Problem?
+        if re.search(r'\w', input_string):
             input_string = re.sub('@', '', input_string)
 
         return input_string
@@ -211,11 +223,10 @@ class Quid:
             for char in minimal_match_character_set:
                 minimal_match_hash.update(char.encode('utf8'))
 
-            hashes.insert(minimal_match_string, minimal_match_hash, False)
-
             if minimal_match_string in min_length_match_starting_positions:
                 min_length_match_starting_positions[minimal_match_string].append(position)
             else:
+                hashes.insert(minimal_match_string, minimal_match_hash, False)
                 min_length_match_starting_positions[minimal_match_string] = [position]
 
         return min_length_match_starting_positions, hashes
@@ -477,7 +488,8 @@ class Quid:
         if min(input1_length, input2_length) < 2:
             return input1 == input2
 
-        ratio = fuzz.ratio(input1, input2)
+        # ratio = fuzz.ratio(input1, input2)
+        ratio = Levenshtein.normalized_similarity(input1, input2)
 
         if max(input1_length, input2_length) < 10:
             return ratio >= self.SCORE_CUTOFF_SHORT_STRINGS
@@ -494,7 +506,8 @@ class Quid:
         if word in candidates:
             return word
 
-        best_candidate = process.extractOne(word, candidates, scorer=fuzz.ratio, score_cutoff=self.SCORE_CUTOFF)
+        best_candidate = process.extractOne(word, candidates, scorer=Levenshtein.normalized_similarity,
+                                            score_cutoff=self.SCORE_CUTOFF)
 
         if best_candidate:
             return best_candidate[0]
@@ -722,17 +735,17 @@ class Quid:
         result = ''
 
         for match in matches:
-            similarity_literature = match.source_span
-            similarity_scientific = match.target_span
+            similarity_literature = match.source_match_span
+            similarity_scientific = match.target_match_span
 
             content = literature_content[
-                      similarity_literature.start:similarity_literature.end]
-            result += f'\n\n{similarity_literature.start}\t{similarity_literature.end}' \
+                      similarity_literature.character_start:similarity_literature.character_end]
+            result += f'\n\n{similarity_literature.character_start}\t{similarity_literature.character_end}' \
                       f'\t{content}'
 
             content = scientific_content[
-                      similarity_scientific.start:similarity_scientific.end]
-            result += f'\n{similarity_scientific.start}\t{similarity_scientific.end}' \
+                      similarity_scientific.character_start:similarity_scientific.character_end]
+            result += f'\n{similarity_scientific.character_start}\t{similarity_scientific.character_end}' \
                       f'\t{content}'
 
         print(result)
