@@ -1,10 +1,11 @@
+from dataclasses import dataclass
 from typing import List
 
 
+@dataclass
 class TargetLocationSelection:
-    def __init__(self, target_text_id: int, target_location_ids: List[int]):
-        self.target_text_id = target_text_id
-        self.target_location_ids = target_location_ids
+    target_text_id: int
+    target_location_ids: List[int]
 
     @classmethod
     def from_value(cls, target_text_id, target_location_id):  # pragma: no cover
